@@ -1,5 +1,5 @@
 ;;; GNU Guix --- Functional package management for GNU
-;;; Copyright © 2012, 2013, 2014, 2015 Ludovic Courtès <ludo@gnu.org>
+;;; Copyright © 2012, 2013, 2014, 2015, 2016 Ludovic Courtès <ludo@gnu.org>
 ;;;
 ;;; This file is part of GNU Guix.
 ;;;
@@ -38,6 +38,7 @@
 
 (use-modules (guix config)
              (guix store)
+             (guix grafts)
              (guix packages)
              (guix derivations)
              (guix monads)
@@ -78,6 +79,9 @@
     (license . ,(package-license package))
     (home-page . ,(package-home-page package))
     (maintainers . ("bug-guix@gnu.org"))
+    (max-silent-time . ,(or (assoc-ref (package-properties package)
+                                       'max-silent-time)
+                            3600))                ; 1 hour by default
     (timeout . ,(or (assoc-ref (package-properties package) 'timeout)
                     72000)))) ; 20 hours by default
 
@@ -157,7 +161,7 @@ system.")
                        (set-guile-for-build (default-guile))
                        (system-disk-image installation-os
                                           #:disk-image-size
-                                          (* 860 MiB))))))
+                                          (* 1024 MiB))))))
       '()))
 
 (define (tarball-jobs store system)
@@ -245,27 +249,35 @@ valid."
                        %packages-to-cross-build))
                 (remove (either from-32-to-64? same?) %cross-targets)))
 
-  ;; Return one job for each package, except bootstrap packages.
-  (append-map (lambda (system)
-                (case subset
-                  ((all)
-                   ;; Build everything.
-                   (fold-packages (lambda (package result)
-                                    (let ((job (package->job store package
-                                                             system)))
-                                      (if job
-                                          (cons job result)
-                                          result)))
-                                  (append (qemu-jobs store system)
-                                          (tarball-jobs store system)
-                                          (cross-jobs system))))
-                  ((core)
-                   ;; Build core packages only.
-                   (append (map (lambda (package)
-                                  (package-job store (job-name package)
-                                               package system))
-                                %core-packages)
-                           (cross-jobs system)))
-                  (else
-                   (error "unknown subset" subset))))
-              %hydra-supported-systems))
+  ;; Turn off grafts.  Grafting is meant to happen on the user's machines.
+  (parameterize ((%graft? #f))
+    ;; Return one job for each package, except bootstrap packages.
+    (append-map (lambda (system)
+                  (case subset
+                    ((all)
+                     ;; Build everything, including replacements.
+                     (let ((all (fold-packages
+                                 (lambda (package result)
+                                   (if (package-replacement package)
+                                       (cons* package
+                                              (package-replacement package)
+                                              result)
+                                       (cons package result)))
+                                 '()))
+                           (job (lambda (package)
+                                  (package->job store package
+                                                system))))
+                       (append (filter-map job all)
+                               (qemu-jobs store system)
+                               (tarball-jobs store system)
+                               (cross-jobs system))))
+                    ((core)
+                     ;; Build core packages only.
+                     (append (map (lambda (package)
+                                    (package-job store (job-name package)
+                                                 package system))
+                                  %core-packages)
+                             (cross-jobs system)))
+                    (else
+                     (error "unknown subset" subset))))
+                %hydra-supported-systems)))
