@@ -18,10 +18,11 @@
 ;;; along with GNU Guix.  If not, see <http://www.gnu.org/licenses/>.
 
 (define-module (gnu packages gprolog)
+  #:use-module (guix download)
   #:use-module (guix licenses)
   #:use-module (guix packages)
-  #:use-module (guix download)
-  #:use-module (guix build-system gnu))
+  #:use-module (guix build-system gnu)
+  #:use-module (srfi srfi-1))
 
 (define-public gprolog
   (package
@@ -52,4 +53,9 @@
 solving over finite domains.  It accepts Prolog+ constraint programs and
 produces a compiled, native binary which can function in a stand-alone
 manner.  It also features an interactive interpreter.")
-    (license (list gpl2+ lgpl3+))))
+    (license (list gpl2+ lgpl3+))
+
+    ;; See 'configure' for the list of supported architectures.
+    (supported-systems (fold delete
+                             %supported-systems
+                             '("armhf-linux" "mips64el-linux")))))
