@@ -2,6 +2,8 @@
 ;;; Copyright © 2014 John Darrington <jmd@gnu.org>
 ;;; Copyright © 2015 Taylan Ulrich Bayırlı/Kammer <taylanbayirli@gmail.com>
 ;;; Copyright © 2015 Mark H Weaver <mhw@netris.org>
+;;; Copyright © 2016 Federico Beffa <beffa@fbengineering.ch>
+;;; Copyright © 2016 Nils Gillmann <niasterisk@grrlz.net>
 ;;;
 ;;; This file is part of GNU Guix.
 ;;;
@@ -27,6 +29,7 @@
   #:use-module (gnu packages texlive)
   #:use-module (gnu packages m4)
   #:use-module (guix download)
+  #:use-module (guix git-download)
   #:use-module (guix utils)
   #:use-module (guix build-system gnu)
   #:use-module (gnu packages base)
@@ -59,15 +62,16 @@
        #:configure-flags '("--enable-ansi") ; required for use by the maxima package
        #:phases (alist-cons-before
                 'configure 'pre-conf
-                (lambda _ 
+                (lambda _
                   ;; Patch bug when building readline support.  This bug was
                   ;; also observed by Debian
                   ;; https://bugs.debian.org/cgi-bin/bugreport.cgi?bug=741819
                   (substitute* "o/gcl_readline.d"
-                    (("rl_attempted_completion_function = \\(CPPFunction \\*\\)rl_completion;")
+                    (("rl_attempted_completion_function = \
+\\(CPPFunction \\*\\)rl_completion;")
                       "rl_attempted_completion_function = rl_completion;"))
-                  (substitute* 
-                      (append 
+                  (substitute*
+                      (append
                        '("pcl/impl/kcl/makefile.akcl"
                          "add-defs"
                          "unixport/makefile.dos"
@@ -115,8 +119,7 @@ interface to the Tk widget system.")
               ("libgc" ,libgc)
               ("libffi" ,libffi)))
     (arguments
-     '(#:tests? #f
-       ;; During 'make check', ECL fails to initialize with "protocol not
+     '(;; During 'make check', ECL fails to initialize with "protocol not
        ;; supported", presumably because /etc/protocols is missing in the
        ;; build environment.  See <http://sourceforge.net/p/ecls/bugs/300/>.
        ;;
@@ -139,7 +142,7 @@ interface to the Tk widget system.")
        ;;         (string-append
        ;;          "ECL=" (assoc-ref outputs "out") "/bin/ecl"))))
        ;;    rearranged-phases))
-       ))
+       #:tests? #f))
     (home-page "http://ecls.sourceforge.net/")
     (synopsis "Embeddable Common Lisp")
     (description "ECL is an implementation of the Common Lisp language as
@@ -215,73 +218,89 @@ an interpreter, a compiler, a debugger, and much more.")
        (sha256
         (base32 "0ab9lw056yf6y0rjmx3iirn5n59pmssqxf00fbmpyl6qsnpaja1d"))))
     (build-system gnu-build-system)
+    (outputs '("out" "doc"))
     ;; Bootstrap with CLISP.
     (native-inputs
      `(("clisp" ,clisp)
        ("which" ,which)
        ("inetutils" ,inetutils)         ;for hostname(1)
-       ("ed" ,ed)))
+       ("ed" ,ed)
+       ("texlive" ,texlive)
+       ("texinfo" ,texinfo)))
     (arguments
      '(#:phases
-       (alist-delete
-        'configure
-        (alist-cons-before
-         'build 'patch-unix-tool-paths
-         (lambda* (#:key outputs inputs #:allow-other-keys)
-           (let ((out (assoc-ref outputs "out"))
-                 (bash (assoc-ref inputs "bash"))
-                 (coreutils (assoc-ref inputs "coreutils"))
-                 (ed (assoc-ref inputs "ed")))
-             (define (quoted-path input path)
-               (string-append "\"" input path "\""))
-             ;; Patch absolute paths in string literals.  Note that this
-             ;; occurs in some .sh files too (which contain Lisp code).  Use
-             ;; ISO-8859-1 because some of the files are ISO-8859-1 encoded.
-             (with-fluids ((%default-port-encoding #f))
-               (substitute* (find-files "." "\\.(lisp|sh)$")
-                 (("\"/bin/sh\"") (quoted-path bash "/bin/sh"))
-                 (("\"/usr/bin/env\"") (quoted-path coreutils "/usr/bin/env"))
-                 (("\"/bin/cat\"") (quoted-path coreutils "/bin/cat"))
-                 (("\"/bin/ed\"") (quoted-path ed "/bin/ed"))
-                 (("\"/bin/echo\"") (quoted-path coreutils "/bin/echo"))
-                 (("\"/bin/uname\"") (quoted-path coreutils "/bin/uname"))))
-             ;; This one script has a non-string occurrence of /bin/sh.
-             (substitute* '("tests/foreign.test.sh")
-               ;; Leave whitespace so we don't match the shebang.
-               ((" /bin/sh ") " sh "))
-             ;; This file contains a module that can create executable files
-             ;; which depend on the presence of SBCL.  It generates shell
-             ;; scripts doing "exec sbcl ..." to achieve this.  We patch both
-             ;; the shebang and the reference to "sbcl", tying the generated
-             ;; executables to the exact SBCL package that generated them.
-             (substitute* '("contrib/sb-executable/sb-executable.lisp")
-               (("/bin/sh") (string-append bash "/bin/sh"))
-               (("exec sbcl") (string-append "exec " out "/bin/sbcl")))
-             ;; Disable some tests that fail in our build environment.
-             (substitute* '("contrib/sb-bsd-sockets/tests.lisp")
-               ;; This requires /etc/protocols.
-               (("\\(deftest get-protocol-by-name/error" all)
-                (string-append "#+nil ;disabled by Guix\n" all)))
-             (substitute* '("contrib/sb-posix/posix-tests.lisp")
-               ;; These assume some users/groups which we don't have.
-               (("\\(deftest pwent\\.[12]" all)
-                (string-append "#+nil ;disabled by Guix\n" all))
-               (("\\(deftest grent\\.[12]" all)
-                (string-append "#+nil ;disabled by Guix\n" all)))))
-         (alist-replace
-          'build
-          (lambda* (#:key outputs #:allow-other-keys)
-            (setenv "CC" "gcc")
-            (zero? (system* "sh" "make.sh" "clisp"
-                            (string-append "--prefix="
-                                           (assoc-ref outputs "out")))))
-          (alist-replace
-           'install
+       (modify-phases %standard-phases
+         (delete 'configure)
+         (add-before 'build 'patch-unix-tool-paths
+           (lambda* (#:key outputs inputs #:allow-other-keys)
+             (let ((out (assoc-ref outputs "out"))
+                   (bash (assoc-ref inputs "bash"))
+                   (coreutils (assoc-ref inputs "coreutils"))
+                   (ed (assoc-ref inputs "ed")))
+               (define (quoted-path input path)
+                 (string-append "\"" input path "\""))
+               ;; Patch absolute paths in string literals.  Note that this
+               ;; occurs in some .sh files too (which contain Lisp code).  Use
+               ;; ISO-8859-1 because some of the files are ISO-8859-1 encoded.
+               (with-fluids ((%default-port-encoding #f))
+                 (substitute* (find-files "." "\\.(lisp|sh)$")
+                   (("\"/bin/sh\"") (quoted-path bash "/bin/sh"))
+                   (("\"/usr/bin/env\"") (quoted-path coreutils "/usr/bin/env"))
+                   (("\"/bin/cat\"") (quoted-path coreutils "/bin/cat"))
+                   (("\"/bin/ed\"") (quoted-path ed "/bin/ed"))
+                   (("\"/bin/echo\"") (quoted-path coreutils "/bin/echo"))
+                   (("\"/bin/uname\"") (quoted-path coreutils "/bin/uname"))))
+               ;; This one script has a non-string occurrence of /bin/sh.
+               (substitute* '("tests/foreign.test.sh")
+                 ;; Leave whitespace so we don't match the shebang.
+                 ((" /bin/sh ") " sh "))
+               ;; This file contains a module that can create executable files
+               ;; which depend on the presence of SBCL.  It generates shell
+               ;; scripts doing "exec sbcl ..." to achieve this.  We patch both
+               ;; the shebang and the reference to "sbcl", tying the generated
+               ;; executables to the exact SBCL package that generated them.
+               (substitute* '("contrib/sb-executable/sb-executable.lisp")
+                 (("/bin/sh") (string-append bash "/bin/sh"))
+                 (("exec sbcl") (string-append "exec " out "/bin/sbcl")))
+               ;; Disable some tests that fail in our build environment.
+               (substitute* '("contrib/sb-bsd-sockets/tests.lisp")
+                 ;; This requires /etc/protocols.
+                 (("\\(deftest get-protocol-by-name/error" all)
+                  (string-append "#+nil ;disabled by Guix\n" all)))
+               (substitute* '("contrib/sb-posix/posix-tests.lisp")
+                 ;; These assume some users/groups which we don't have.
+                 (("\\(deftest pwent\\.[12]" all)
+                  (string-append "#+nil ;disabled by Guix\n" all))
+                 (("\\(deftest grent\\.[12]" all)
+                  (string-append "#+nil ;disabled by Guix\n" all))))))
+         (replace 'build
+           (lambda* (#:key outputs #:allow-other-keys)
+             (setenv "CC" "gcc")
+             (zero? (system* "sh" "make.sh" "clisp"
+                             (string-append "--prefix="
+                                            (assoc-ref outputs "out"))))))
+         (replace 'install
            (lambda _
-             (zero? (system* "sh" "install.sh")))
-           %standard-phases))))
-       ;; No 'check' target, though "make.sh" (build phase) runs tests.
-       #:tests? #f))
+             (zero? (system* "sh" "install.sh"))))
+         (add-after 'build 'build-doc
+           (lambda _
+             (with-directory-excursion "doc/manual"
+               (and  (zero? (system* "make" "info"))
+                     (zero? (system* "make" "dist"))))))
+         (add-after 'install 'install-doc
+           (lambda* (#:key outputs #:allow-other-keys)
+             (let* ((out (assoc-ref outputs "out"))
+                    (doc (assoc-ref outputs "doc"))
+                    (old-doc-dir (string-append out "/share/doc"))
+                    (new-doc/sbcl-dir (string-append doc "/share/doc/sbcl")))
+               (rmdir (string-append old-doc-dir "/sbcl/html"))
+               (mkdir-p new-doc/sbcl-dir)
+               (copy-recursively (string-append old-doc-dir "/sbcl")
+                                 new-doc/sbcl-dir)
+               (delete-file-recursively old-doc-dir)
+               #t))))
+         ;; No 'check' target, though "make.sh" (build phase) runs tests.
+         #:tests? #f))
     (home-page "http://www.sbcl.org/")
     (synopsis "Common Lisp implementation")
     (description "Steel Bank Common Lisp (SBCL) is a high performance Common
@@ -408,3 +427,49 @@ interface.")
     ;; applies to Lisp code according to them.
     (license (list license:lgpl2.1
                    license:clarified-artistic)))) ;TRIVIAL-LDAP package
+
+(define-public lispf4
+  (let ((commit "174d8764d2f9764e8f4794c2e3feada9f9c1f1ba"))
+    (package
+      (name "lispf4")
+      (version (string-append "0.0.0-1" "-"
+                              (string-take commit 7)))
+      (source (origin
+                (method git-fetch)
+                (uri (git-reference
+                      (url "https://github.com/blakemcbride/LISPF4.git")
+                      (commit commit)))
+                (file-name (string-append name "-" version))
+                (sha256
+                 (base32
+                  "18k8kfn30za637y4bfbm9x3vv4psa3q8f7bi9h4h0qlb8rz8m92c"))))
+      (build-system gnu-build-system)
+      ;; 80 MB appended Documentation -> output:doc
+      (outputs '("out" "doc"))
+      (arguments
+       `(#:make-flags
+         '("-f" "Makefile.unx" "CC=gcc")
+         ;; no check phase
+         #:tests? #f
+         #:phases
+         (modify-phases %standard-phases
+           (delete 'configure)
+           (replace
+            'install
+            (lambda* (#:key outputs inputs #:allow-other-keys)
+              (let* ((out (assoc-ref outputs "out"))
+                     (bin (string-append out "/bin"))
+                     (doc (string-append (assoc-ref outputs "doc")
+                                         "/share/doc/lispf4")))
+                (install-file "lispf4" bin)
+                (install-file "SYSATOMS" bin)
+                (install-file "BASIC.IMG" bin)
+                (copy-recursively "Documentation" doc))
+                #t)))))
+      (synopsis "InterLisp interpreter")
+      (description
+       "LISPF4 is an InterLisp interpreter written in FORTRAN by Mats Nordstrom
+in the early 80's.  It was converted to C by Blake McBride and supports much of
+the InterLisp Standard.")
+      (home-page "https://github.com/blakemcbride/LISPF4.git")
+      (license license:expat))))
