@@ -1,5 +1,6 @@
 ;;; GNU Guix --- Functional package management for GNU
-;;; Copyright © 2013, 2014 Ludovic Courtès <ludo@gnu.org>
+;;; Copyright © 2013, 2014, 2015 Ludovic Courtès <ludo@gnu.org>
+;;; Copyright © 2015, 2016 Mark H Weaver <mhw@netris.org>
 ;;;
 ;;; This file is part of GNU Guix.
 ;;;
@@ -33,28 +34,54 @@
   #:use-module (gnu packages image)
   #:use-module (gnu packages attr)
   #:use-module (gnu packages linux)
-  #:use-module (gnu packages xorg)
+  #:use-module (gnu packages libusb)
+  #:use-module (gnu packages xdisorg)
   #:use-module (gnu packages gl)
   #:use-module (gnu packages sdl)
-  #:use-module (gnu packages perl))
+  #:use-module (gnu packages perl)
+  #:use-module (srfi srfi-1))
 
-(define-public qemu-headless
-  ;; This is QEMU without GUI support.
+(define (qemu-patch commit file-name sha256)
+  "Return an origin for COMMIT."
+  (origin
+    (method url-fetch)
+    (uri (string-append
+          "http://git.qemu.org/?p=qemu.git;a=commitdiff_plain;h="
+          commit))
+    (sha256 sha256)
+    (file-name file-name)))
+
+(define-public qemu
   (package
-    (name "qemu-headless")
-    (version "2.0.0")
+    (name "qemu")
+    (version "2.5.0")
     (source (origin
              (method url-fetch)
              (uri (string-append "http://wiki.qemu-project.org/download/qemu-"
                                  version ".tar.bz2"))
              (sha256
               (base32
-               "0frsahiw56jr4cqr9m6s383lyj4ar9hfs2wp3y4yr76krah1mk30"))))
+               "1m3j6xl7msrniidkvr5pw9d44yba5m7hm42xz8xy77v105s8hhrl"))
+             (patches
+              (map search-patch
+                   '("qemu-virtio-9p-use-accessor-to-get-thread-pool.patch"
+                     "qemu-CVE-2015-8558.patch"
+                     "qemu-CVE-2015-8567.patch"
+                     "qemu-CVE-2016-1922.patch"
+                     "qemu-CVE-2015-8613.patch"
+                     "qemu-CVE-2015-8701.patch"
+                     "qemu-CVE-2015-8743.patch"
+                     "qemu-CVE-2016-1568.patch"
+                     "qemu-CVE-2015-8619.patch"
+                     "qemu-CVE-2016-1981.patch"
+                     "qemu-usb-ehci-oob-read.patch"
+                     "qemu-CVE-2016-2197.patch")))))
     (build-system gnu-build-system)
     (arguments
      '(#:phases (alist-replace
                  'configure
-                 (lambda* (#:key inputs outputs #:allow-other-keys)
+                 (lambda* (#:key inputs outputs (configure-flags '())
+                           #:allow-other-keys)
                    ;; The `configure' script doesn't understand some of the
                    ;; GNU options.  Thus, add a new phase that's compatible.
                    (let ((out (assoc-ref outputs "out")))
@@ -67,11 +94,13 @@
                      ;; The binaries need to be linked against -lrt.
                      (setenv "LDFLAGS" "-lrt")
                      (zero?
-                      (system* "./configure"
-                               (string-append "--cc=" (which "gcc"))
+                      (apply system*
+                             `("./configure"
+                               ,(string-append "--cc=" (which "gcc"))
                                "--disable-debug-info" ; save build space
                                "--enable-virtfs"      ; just to be sure
-                               (string-append "--prefix=" out)))))
+                               ,(string-append "--prefix=" out)
+                               ,@configure-flags)))))
                  (alist-cons-after
                   'install 'install-info
                   (lambda* (#:key inputs outputs #:allow-other-keys)
@@ -87,10 +116,23 @@
                                               (string-append infodir "/" info)))
                                            (find-files "." "\\.info$"))
                                  #t)))))
-                  %standard-phases))))
+                  (alist-cons-before
+                   'check 'disable-test-qga
+                   (lambda _
+                     (substitute* "tests/Makefile"
+                       ;; Comment out the test-qga test, which needs /sys and
+                       ;; fails within the build environment.
+                       (("check-unit-.* tests/test-qga" all)
+                        (string-append "# " all)))
+                     #t)
+                   %standard-phases)))))
 
     (inputs                                       ; TODO: Add optional inputs.
-     `(;; ("libaio" ,libaio)
+     `(("sdl" ,sdl)
+       ("mesa" ,mesa)
+       ("libusb" ,libusb)                         ;USB pass-through support
+
+       ;; ("libaio" ,libaio)
        ("glib" ,glib)
        ("ncurses" ,ncurses)
        ("libpng" ,libpng)
@@ -110,7 +152,7 @@
                      ("texinfo" ,texinfo)
                      ("perl" ,perl)))
     (home-page "http://www.qemu-project.org")
-    (synopsis "Machine emulator and virtualizer (without GUI)")
+    (synopsis "Machine emulator and virtualizer")
     (description
      "QEMU is a generic machine emulator and virtualizer.
 
@@ -130,11 +172,17 @@ server and embedded PowerPC, and S390 guests.")
     ;; Several tests fail on MIPS; see <http://hydra.gnu.org/build/117914>.
     (supported-systems (delete "mips64el-linux" %supported-systems))))
 
-(define-public qemu
-  ;; QEMU with GUI support.
-  (package (inherit qemu-headless)
-    (name "qemu")
-    (synopsis "Machine emulator and virtualizer")
-    (inputs `(("sdl" ,sdl)
-              ("mesa" ,mesa)
-              ,@(package-inputs qemu-headless)))))
+(define-public qemu-minimal
+  ;; QEMU without GUI support.
+  (package (inherit qemu)
+    (name "qemu-minimal")
+    (synopsis "Machine emulator and virtualizer (without GUI)")
+    (arguments
+     `(#:configure-flags
+       ;; Restrict to the targets supported by Guix.
+       '("--target-list=i386-softmmu,x86_64-softmmu,mips64el-softmmu,arm-softmmu")
+       ,@(package-arguments qemu)))
+
+    ;; Remove dependencies on optional libraries, notably GUI libraries.
+    (inputs (fold alist-delete (package-inputs qemu)
+                  '("sdl" "mesa" "libusb")))))
