@@ -1,5 +1,5 @@
 ;;; GNU Guix --- Functional package management for GNU
-;;; Copyright © 2013 Ludovic Courtès <ludo@gnu.org>
+;;; Copyright © 2013, 2015 Ludovic Courtès <ludo@gnu.org>
 ;;; Copyright © 2015 Mark H Weaver <mhw@netris.org>
 ;;;
 ;;; This file is part of GNU Guix.
@@ -32,14 +32,14 @@
 (define swig
   (package
     (name "swig")
-    (version "3.0.2")
+    (version "3.0.5")
     (source (origin
              (method url-fetch)
              (uri (string-append "mirror://sourceforge/swig/swig-"
                                  version ".tar.gz"))
              (sha256
               (base32
-               "04vqrij3k6pcq41y7rzl5rmhnghqg905f11wyrqw7vdwr9brcrm2"))))
+               "0g1a69vrqxgsnr1wkx851ljn73a2x3jqzxa66s2l3w0kyblbjk4z"))))
     (build-system gnu-build-system)
     (native-inputs `(("boost" ,boost)))
     (inputs `(("pcre" ,pcre)
@@ -54,18 +54,6 @@
               ;; The python part probably never worked and does not seem to
               ;; be needed for currently dependent packages.
 ;;               ("python" ,python-wrapper)))
-    (arguments
-     `(#:phases
-       (alist-cons-before
-        'check 'install-locales
-        (lambda _
-          ;; One of the tests requires the availability of a UTF-8
-          ;; locale and otherwise fails.
-          (setenv "LOCPATH" (getcwd))
-          (zero? (system* "localedef" "--no-archive"
-                          "--prefix" (getcwd) "-i" "en_US"
-                          "-f" "UTF-8" "./en_US.utf8")))
-        %standard-phases)))
     (home-page "http://swig.org/")
     (synopsis
      "Interface compiler that connects C/C++ code to higher-level languages")
