@@ -29,37 +29,45 @@
 (define-public zsh
   (package
     (name "zsh")
-    (version "5.0.7")
+    (version "5.1.1")
     (source (origin
-             (method url-fetch)
-             (uri (string-append "http://www.zsh.org/pub/zsh-" version
-                                 ".tar.gz"))
-             (sha256
-              (base32 "06drwywxa0zm8mizk32wc6ijhpzlyy2mcmskj66bnyggg70s9w23"))))
+              (method url-fetch)
+              (uri (list (string-append
+                           "http://www.zsh.org/pub/zsh-" version
+                           ".tar.gz")
+                         (string-append
+                           "http://www.zsh.org/pub/old/zsh-" version
+                           ".tar.gz")))
+              (sha256
+               (base32
+                "11shllzhq53fg8ngy3bgbmpf09fn2czifg7hsb41nxi3410mpvcl"))))
     (build-system gnu-build-system)
     (arguments `(#:configure-flags '("--with-tcsetpgrp" "--enable-pcre")
                  #:phases (alist-cons-before
                            'configure 'fix-sh
                            (lambda _
-                             (substitute*
-                                 '("configure"
-                                   "configure.ac"
-                                   "Src/exec.c"
-                                   "Src/mkmakemod.sh"
-                                   "Config/installfns.sh"
-                                   "Config/defs.mk.in"
-                                   "Test/E01options.ztst"
-                                   "Test/A05execution.ztst"
-                                   "Test/A01grammar.ztst"
-                                   "Test/B02typeset.ztst"
-                                   "Completion/Unix/Command/_init_d"
-                                   "Util/preconfig")
-                               (("/bin/sh") (which "sh"))))
+                             ;; Some of the files are ISO-8859-1 encoded.
+                             (with-fluids ((%default-port-encoding #f))
+                               (substitute*
+                                   '("configure"
+                                     "configure.ac"
+                                     "Src/exec.c"
+                                     "Src/mkmakemod.sh"
+                                     "Config/installfns.sh"
+                                     "Config/defs.mk.in"
+                                     "Test/E01options.ztst"
+                                     "Test/A05execution.ztst"
+                                     "Test/A01grammar.ztst"
+                                     "Test/A06assign.ztst"
+                                     "Test/B02typeset.ztst"
+                                     "Completion/Unix/Command/_init_d"
+                                     "Util/preconfig")
+                                 (("/bin/sh") (which "sh")))))
                            %standard-phases)))
-    (native-inputs `(("autoconf", autoconf)))
-    (inputs `(("ncurses", ncurses)
-              ("pcre", pcre)
-              ("perl", perl)))
+    (native-inputs `(("autoconf" ,autoconf)))
+    (inputs `(("ncurses" ,ncurses)
+              ("pcre" ,pcre)
+              ("perl" ,perl)))
     (synopsis "Powerful shell for interactive use and scripting")
     (description "The Z shell (zsh) is a Unix shell that can be used
 as an interactive login shell and as a powerful command interpreter
