@@ -137,6 +137,13 @@ bool Settings::get(const string & name, bool def)
     return res;
 }
 
+int Settings::get(const string & name, int def)
+{
+    int res = def;
+    _get(res, name);
+    return res;
+}
+
 
 void Settings::update()
 {
