@@ -1,5 +1,5 @@
 ;;; GNU Guix --- Functional package management for GNU
-;;; Copyright © 2013, 2014, 2015 Ludovic Courtès <ludo@gnu.org>
+;;; Copyright © 2013, 2014, 2015, 2016 Ludovic Courtès <ludo@gnu.org>
 ;;; Copyright © 2014 Alex Kost <alezost@gmail.com>
 ;;;
 ;;; This file is part of GNU Guix.
@@ -22,6 +22,7 @@
   #:use-module (guix profiles)
   #:use-module (guix store)
   #:use-module (guix monads)
+  #:use-module (guix grafts)
   #:use-module (guix packages)
   #:use-module (guix derivations)
   #:use-module (guix build-system trivial)
@@ -32,6 +33,7 @@
   #:use-module (ice-9 regex)
   #:use-module (ice-9 popen)
   #:use-module (rnrs io ports)
+  #:use-module (srfi srfi-1)
   #:use-module (srfi srfi-11)
   #:use-module (srfi srfi-64))
 
@@ -40,6 +42,9 @@
 (define %store
   (open-connection-for-tests))
 
+;; Globally disable grafts because they can trigger early builds.
+(%graft? #f)
+
 (define-syntax-rule (test-assertm name exp)
   (test-assert name
     (run-with-store %store exp
@@ -224,6 +229,14 @@
                            (package-native-search-paths
                             packages:guile-2.0)))))))))
 
+(test-assert "package->manifest-entry, search paths"
+  ;; See <http://bugs.gnu.org/22073>.
+  (let ((mpl (@ (gnu packages python) python2-matplotlib)))
+    (lset= eq?
+           (package-transitive-native-search-paths mpl)
+           (manifest-entry-search-paths
+            (package->manifest-entry mpl)))))
+
 (test-assertm "etc/profile"
   ;; Make sure we get an 'etc/profile' file that at least defines $PATH.
   (mlet* %store-monad
